/*
 * qt: quick and dirty terminal program written for the Atari ST.
 *
 * cmd.c -- routines to handle all user commands.
 *
 * revisions:
 *
 * 11-jan-87 orc:	functions split off from qt.c
 * 12-jan-87 orc:	diddle format for showmacro()
 * 14-jan-87 orc:	if $SHELL not found, shellescape uses \command.prg
 * 21-jan-87 orc:	add verbose flag stuff
 * 23-jan-87 orc:	add carrier detection.
 * 23-jan-87 orc:	Remove $SHELL for shellescape -- use lpexec() instead.
 * 21-feb-87 orc:	have chatterbox strip sign bit from modem input.
 *			make connect escape character configurable.
 */

#include "qt.h"

extern char num[CSIZE],	/* last phone number called (for redialing) */
	    *note;	/* comments on the last number (host name, et. al.) */
extern int  baud;	/* initial baudrate */

extern char *phone[];	/* phonebook ('a'-'z') */
extern char *macro[];	/* macroization of each key */

		    /* settable things */

extern char cfgfile[];
extern char prefix[];
extern char suffix[];

extern int imageMode;	/* do file transfers in binary mode */
extern int timeOut;	/* thirty second timeout */
extern int startConn,	/* connect when we start up? */
	   startDial,	/* dial somebody when we start up */
	   verbose;	/* chatter on and on and on... */
extern int ESCAPE;	/* connect mode escape character */



shellescape()
/* run a subprocess */
{
    long status, lpexec();
    char tail[128];
    char cmd[128];
    char *sp, *tp, *cp, *getenv();

    conPut('!'); getString(tail,127);		/* get command to execute   */
    for (tp=tail;*tp && *tp <= ' '; tp++)	/* split it into name, tail */
	;
    for (sp=cmd;*tp > ' '; *sp++ = *tp++)
	;
    if (sp == cmd)				/* run a subshell? */
	strcpy(cmd,(cp = getenv("SHELL"))?cp:"\\command.prg");
    else
	*sp = 0;
    while (*tp && *tp <= ' ')			/* find start of next arg */
	tp++;
    if (*--tp)					/* add length byte to tail */
	*tp = strlen(1+tp);
    else					/* or assure a null tail */
	(++tp)[1] = 0;
    
    status = lpexec(cmd, tp, NULL);	/* do it */
    if (status < 0) {
	printString((status == -33L)?"file not found.":"Permission denied.");
	println();
    }
} /* shellescape */



directory()
/* get a directory listing */
{
    char s[80];
    int size;			/* for number formatting */
    long kfree;			/* kbytes free on this device */
    int drv = 0;		/* drive we did the directory on... */
    int xp = 0;

    struct {			/* dta struct used by dfirst()/dnext() */
	char d_rsvd[21];
	char d_attr;
	long d_timestamp;
	long d_size;
	char d_name[15];
    } dta;

    struct {			/* parameter block returned by dfree() */
	long b_free;
	long b_total;
	long b_secsiz;
	long b_clsiz;
    } drvbuf;

    printString("dir ");getString(s,79);
    if (!s[0])
	strcpy(s,"*.*");
    else if (s[strlen(s)-1] == ':')
	strcat(s,"\\*.*");
    else if (s[strlen(s)-1] == '\\')
	strcat(s,"*.*");

    if (s[1] == ':')
	drv = ((s[0] | 0x20) - 'a') + 1;

    dsetdta(&dta);
    if (dfirst(s,0x1F)) {
	if (strchr(s,'?') || strchr(s,'*'))
	    printString("no files found.");
	else
	    printString("no such file.");
	println();
    }
    else {
	do {
	    printChar((dta.d_attr & 0x18)?'d':'-');
	    printChar((dta.d_attr & 0x01)?'-':'w');
	    printChar((dta.d_attr & 0x02)?'s':'-');
	    printChar((dta.d_attr & 0x04)?'h':'-');
	    printChar(' ');
	    if (dta.d_attr & 0x18)
		printString("     ");
	    else
		for (size=printNum(1+(int)(dta.d_size/1024)); size<5; size++)
		    printChar(' ');
	    size = printString(dta.d_name);
	    if (xp == 2)
		println();
	    else
		while (size++ < 17)
		    printChar(' ');
	    xp = (1+xp)%3;
	} while (dnext() == 0);
    }
    if (xp)
	println();
    dfree(drv, &drvbuf);
    kfree = (drvbuf.b_free * drvbuf.b_secsiz * drvbuf.b_clsiz) / 1024L;
    if (drv) {
	printChar((drv+'a')-1);
	printString(": ");
    }
    printNum((int)(kfree));
    printString(" free kbyte");
    if (kfree != 1)
	printChar('s');
    printChar('.');
    println();
} /* directory */



setbaud()
/* set or show the baudrate */
{
    unsigned c;
    int change = 0;

    conString("baud ");
    printChar(c = getKey());

    switch (c) {
    case '3':
	printString("00");
	baud = B_300;
	change++;
	break;
    case '2':
	printString("400");
	baud = B_2400;
	change++;
	break;
    case '1':
	printString("200");
	baud = B_1200;
	change++;
	break;
    case '=':
	switch (baud) {
	case B_300:
	    printString(" 300");
	    break;
	case B_1200:
	    printString(" 1200");
	    break;
	case B_2400:
	    printString(" 2400");
	    break;
	}
	break;
    case '?':
	println();
	printString("1 - set 1200 baud.");println();
	printString("2 - set 2400 baud.");println();
	printString("3 - set 300 baud.");println();
	printString("= - show current baudrate.");
	break;
    default:
	conPut(7);
	if (verbose)
	    printString("? -- type 'b?' for help.");
    case ESC:
	break;
    }
    println();
    if (change)
	rsconf(baud, 0, -1, -1, -1, -1);
} /* setbaud */



hangup()
/* forcibly drop carrier */
{
    register long i;
    int idle;

    if (idle = noCarrier())		/* if there is no carrier (dialing) */
	auxPut('\r');			/*   just kick the modem once..     */

    ongibit(0x10);			/* Zot DTR */

    if (idle)				/* can't register carrier loss, so  */
	for (i=(1L<<16); (i > 0); --i)	/* just burn a little time          */
	    ;
    else				/* online, so wait til carrier drops*/
					/* or a 2^19 loop counts down...    */
	for (i = (1L<<19); (i > 0) && !noCarrier(); --i)
	    ;

    offgibit(0xffef);			/* turn DTR back on */
} /* hangup */



chatterbox()
/* talk to the modem... */
{
    register unsigned c;

    while (1) {
	if (conStat()) {
	    c = getKey();
	    if (c == ESCAPE)
		break;
	    if (macro[c])
		auxString(macro[c]);
	    else
		auxPut(c);
	}
	if (auxStat())
	    conPut(auxGet()&0x7F);
    }
} /* chatterbox */



dial()
/* autodialing submenu */
{
    unsigned c;
    unsigned retval;

    conPut('p');
    c = getKey();
    conPut(8);

    if (c >= 'a' && c <= 'z')
	return dialnum(c);
    else if (c == '=') {
	conPut(' ');
	println();
	for (c='a'; c <= 'z'; c++)
	    if (phone[c-'a']) {
		printChar(c);
		conString(" =>\t");
		printString(phone[c-'a']);
		println();
	    }
    }
    else if (c == ';') {
	printString("call what number? ");getString(num,79);
	retval = ringup(num,NULL,note=NULL);
	println();
	return retval;
    }
    else if (c == '!') {
	if (!num[0]) {
	    printString("nothing to redial.");
	    println();
	}
	else {
	    return ringup(num,"calling ",note);
	}
    }
    else if (c == '?') {
	println();
	printString("a-z - call phone #a-z.");println();
	printString(";   - call another number.");println();
	printString("!   - redial last number.");println();
	printString("=   - list the phonebook.");println();
    }
    else {
	if (verbose)
	    printString("? -- type 'p?' for help.");
	conPut(' ');
	println();
    }
    return FALSE;
}



showoptions()
/* show all the current option settings */
{
    println();
    if (!noCarrier()) {
	printString("*Connected to host.");
	println();
	println();
    }
    printString("escape  = ");printChar(ESCAPE);println();
    printString(imageMode?"image":"text");printString(" xfers.");println();
    printString(verbose?"verbose":"terse");println();
    printString("timeout = ");printNum(timeOut);println();
    printString("prefix  = ");printString(prefix);println();
    printString("suffix  = ");printString(suffix);println();
} /* showoptions */



showmacros()
/* show all the macros */
{
    int c;

    println();
    for (c=0;c<0x100;c++)
	if (macro[c]) {
	    printChar(c);
	    conString("\t= ");
	    printString(macro[c]);
	    println();
	}
} /* showmacros */





dotcommand()
/* set a option, add a phone # to the phonebook, or create a macro */
{
    unsigned c;
    char line[80];

    conPut('.');
    c=getKey();

    switch (toUpper(c)) {
    case 'B':
	printChar(c);
	println();
	printString(" QT was written Dec-86 by Orc (David Parsons)");println();
	printString(" This is version ");printString(VERSION);println();
	break;
    case 'M':
	printString("map: ");getString(line,79);
	if (line[0])
	    addmacro(line);
	break;
    case 'P':
	printString("phone number: ");getString(line,79);
	if (line[0])
	    addphone(line);
	break;
    case 'S':
	printString("set: ");getString(line,79);
	if (line[0])
	    setoption(line);
	break;
    case 'R':
	printString("read cfg? ");getString(line,79);
	if (line[0] && !getcfg(line)) {
	    printString("couldn't read ");printString(line);println();
	}
	break;
    case 'W':
	printString("write cfg? ");getString(line,79);
	if (line[0] && !putcfg(line)) {
	    printString("couldn't write ");printString(line);println();
	}
	break;
    case '?':
	printChar(c);
	println();
	printString(".b  - display banner.");println();
	printString(".m  - define a macro.");println();
	printString(".p  - add a phone number to the phonebook.");println();
	printString(".r  - read in a configuration file.");println();
	printString(".s  - set an option.");println();
	printString(".w  - write current configuration.");println();
	break;
    default:
	conPut(7);
	if (verbose)
	    printString("? -- type '.?' for help.");
    case ESC:
	println();
	break;
    }
} /*dotcommand*/



command()
/* execute commands until I want to quit */
{
    unsigned c;
    int prompt = TRUE;

    do {
	if (prompt)
	    printString(noCarrier()?"Qt>":"*Qt>");
	else prompt = TRUE;

	switch (c = toUpper(getKey())) {
	case 'B':
	    setbaud();
	    break;
	case 'C':
	    if (verbose)
		printString("(connect)");
	    println();
	    chatterbox();
	    break;
	case 'D':
	    directory();
	    break;
	case 'H':
	    printString("(thud)");
	    hangup();
	    println();
	    break;
	case 'M':
	    showmacros();
	    break;
	case 'O':
	    showoptions();
	    break;
	case 'P':
	    if (dial())
		chatterbox();
	    break;
	case 'Q':
	    break;
	case 'X':
	    xmodem();
	    break;
	case '.':
	    dotcommand();
	    break;
	case '!':
	    shellescape();
	    break;
	case '?':
	    clearScreen();
	    printString("b - set the baud rate.");println();
	    printString("c - connect.");println();
	    printString("d - get a directory listing.");println();
	    printString("h - hang up the phone.");println();
	    printString("m - show macros.");println();
	    printString("o - show options.");println();
	    printString("p - call a number.");println();
	    printString("q - quit.");println();
	    printString("x - xmodem file transfer.");println();
	    printString(". - extended commands.");println();
	    printString("! - shell escape.");println();
	    break;
	default:
	    conPut(7);
	    if (verbose) {
		printString("? -- type '?' for help.");
		println();
	    }
	    else
		prompt = FALSE;
	    break;
	}
    } while (c != 'Q');
} /* command */
