/*
 * qt: quick and dirty terminal program written for the Atari ST.
 *
 * qt.c -- main, initialization, and subroutines.
 *
 * revisions:
 *
 * 15-dec-86 orc:	written.
 * 10-jan-87 orc:	xmodem upload/download added & commented.
 * 11-jan-87 orc:	added directory(), . commands.
 * 14-jan-87 orc:	convert auxGet(), auxPut(), conGet() into assembly.
 * 19-jan-87 orc:	Remove protected memory diddles.
 * 21-jan-87 orc:	add verbose flag.
 * 23-jan-87 orc:	add carrier detection functions.
 * 25-jan-87 orc:	add size parameter to getString()
 *			& add toUpper() function.
 * 21-feb-87 orc:	ESCAPE made user-configurable.
 */

#include "qt.h"

char num[CSIZE],	/* last phone number called (for redialing) */
     *note= NULL;	/* comments on the last number (host name, et. al.) */
int  baud = B_1200;	/* initial baudrate */

char *phone[26];	/* phonebook ('a'-'z') */
char *macro[256];	/* macroization of each key */

		    /* settable things */

char cfgfile[CSIZE]= "qt.rc";	/* configuration file to read when starting up */
char prefix[CSIZE] = "ATD";	/* to send to the modem when autodialing */
char suffix[CSIZE] = "\r";	/*  "   "   "  */

int imageMode = TRUE;	/* do file transfers in binary mode */
int timeOut   = 30;	/* thirty second timeout */
int startConn = FALSE,	/* connect when we start up? */
    startDial = -1,	/* dial somebody when we start up */
    verbose   = TRUE;	/* chatter on and on and on */
int ESCAPE    = 0xE1;	/* connect mode escape character */

	    /* my very own xtoa/atox stuff */

char hexdig[] = "0123456789abcdefABCDEF";
#define isHex(c) strchr(hexdig,(c))



/***************************************************************************/



long
conGet()
/* get raw input from the keyboard */
{
    asm("move.w #2, -(sp)");	/* get input from */
    asm("move.w #2, -(sp)");	/* the console */
    asm("trap   #13");
    asm("addq.l #4, sp");
} /* conGet */



conPut(c)
/* absurdly fast console output */
/* abandon all hope ye who debug here */
register int c;
{
    if (c < ' ') {
	asm(" move.w  r7, -(sp)");
	asm(" move.w  #2, -(sp)");
    }
    else {
	asm(" move.w  r7, -(sp)");
	asm(" move.w  #5, -(sp)");
    }
    asm(" move.w  #3, -(sp)");
    asm(" trap    #13");
    asm(" addq.l  #6, sp");
} /* conPut */



unsigned
auxGet()
/* get raw input from the modem port */
{
    asm("move.w #1,-(sp)");
    asm("move.w #2,-(sp)");
    asm("trap   #13");
    asm("addq.l #4,sp");
} /* auxGet */



auxPut(c)
register unsigned c;
{
    asm("move.w  r7, -(sp)");
    asm("move.w  #1, -(sp)");
    asm("move.w  #3, -(sp)");
    asm("trap    #13");
    asm("addq.l  #6, sp");
} /* auxPut */



conStat()
/* get console status */
{
    asm("move.w  #2, -(sp)");
    asm("move.w  #1, -(sp)");
    asm("trap    #13");
    asm("addq.l  #4, sp");
} /* conStat */



auxStat()
/* get modem port status */
{
    asm("move.w  #1, -(sp)");
    asm("move.w  #1, -(sp)");
    asm("trap    #13");
    asm("addq.l  #4, sp");
} /* auxStat */



static int CDmfp;	/* last mfp read by CDcore */

static
CDcore()
/* supervisor mode core for noCarrier() */
{
    CDmfp = *MFP;	/* all it does is get the MFP register from highcore */
} /* CDcore */



noCarrier()
/* get carrier status and return TRUE if none. */
{
    asm(" pea    _CDcore");		/* call CDcore in supervisor mode  */
    asm(" move.w #38, -(sp)");		/* to avoid nasty little mushrooms.*/
    asm(" trap   #14");
    asm(" addq.l #6, sp");

    return CDmfp & 02;			/* bit 1 set when carrier is low */
} /* noCarrier */



/***************************************************************************/



unsigned
getKey()
/* get input from the keyboard. All odd keys (function keys, et al) that
 * do not produce a character have their scancode orred with $80 and returned.
 */
{
    unsigned c;
    long key;

    c = (key = conGet()) & 0xff;
    if (!c)
	return (((unsigned)(key>>16))|0x80) & 0xff;
    return c;
} /* getKey */



auxFlush()
/* throw away all pending input from the modem port */
{
    while (auxStat())
	auxGet();
} /* auxFlush */



auxString(s)
/* write a string to the modem port */
char *s;
{
    while (*s) {
	auxPut(*s++);
	if (auxStat())
	    conPut(auxGet());
    }
} /* auxString */



char _whiteSpace[] = "\t\f ";
#define inWhite(p) strchr(_whiteSpace,p)



getString(s,size)
/* get a string from the console */
char *s;
{
    register char *p = s;
    register unsigned c;
    int mysize = 0;

    while ((c = getKey()) != '\r') {
	if (c == ('H'-'@') ) {			/* erase a character */
	    if (p > s) {
		conString("\010 \010");
		--mysize,--p;
	    }
	}
	else if (c == ('W'-'@') ) {		/* erase a word */
	    while (p > s && !inWhite(p[-1])) {
		conString("\010 \010");
		--mysize,--p;
	    }
	    while (p > s && inWhite(p[-1])) {
		conString("\010 \010");
		--mysize,--p;
	    }
	}
	else if (c == ('U'-'@') ) {		/* erase everything */
	    while (p > s) {
		conString("\010 \010");
		--mysize,--p;
	    }
	}
	else if (mysize < size) {
	    mysize++;
	    *p++ = c;
	    if (c >= ' ' && c <= '~')
		conPut(c);
	    else
		conPut('@');
	}
	else
	    conPut(7);
    }
    *p = 0;
    println();
} /* getString */



ringup(number,msg1,msg2)
/* tell the modem to autodial a number */
char *number;
char *msg1,*msg2;
{
    register cs;

    auxFlush();				/* clear pending input */
    if (msg1 && msg2) {			/* print whatever messages */
	printString(msg1);
	printString(msg2);
	if (verbose)
	    printString(" -- ");
    }
    if (verbose)
	printString("type anything to cancel.");

    auxString(prefix);			/* tell the modem to dial */
    auxString(number);
    auxString(suffix);

    while (noCarrier() && (cs = conStat()) == 0)/* wait for carrier  */
	;					/* or cancel         */
    if (cs) {				/* got a character -- cancel.*/
	conGet();			/* so eat this character     */
	hangup();			/* and drop carrier.         */
	if (verbose) {
	    println();
	    printString("(cancelled)");
	}
    }
    return (cs == 0);
} /* ringup */



dialnum(c)
/* call phone # c */
unsigned c;
{
    int retval;

    if (phone[c-'a']) {
	strcpy(num,phone[c-'a']);
	note = strchr(num,':');
	if (note && *note)
	    *note++ = 0;
	else
	    note = NULL;
	retval = ringup(num,"calling ",note);
	println();
	return retval;
    }
    else {
	conPut(c);
	printString(": not in phonebook.");
	println();
	return FALSE;
    }
} /* dialnum */



static char fkey[] = "<f?>";
static char hc[] = "\\xx";



conString(s)
/* print a string to the console */
register char *s;
{
    while (*s)
	conPut(*s++);
} /* conString */



clearScreen()
/* clear the screen */
{
    bios(3,2,033);
    bios(3,2,'E');
} /* clear the screen */



char *
printable(s)
/* convert a string into a human-readable form */
register char *s;
{
    static char lb[CSIZE+3];
    register char *lbp;
    unsigned c;
 
    for (lbp=lb; *s; s++)
	if (*s >= ' ' && *s <= '~')
	    *lbp++ = *s;
	else {
	    *lbp++ = '\\';
	    switch (c = (*s)&0xff) {
	    case '\n':
		*lbp++ = 'n';
		break;
	    case '\r':
		*lbp++ = 'r';
		break;
	    case '\t':
		*lbp++ = 't';
		break;
	    default:
		*lbp++ = hexdig[c>>4];
		*lbp++ = hexdig[c&15];
		break;
	    }
	}
    *lbp = 0;
    return lb;
} /* printable */



printChar(c)
/* print out a character in human-readable form */
int c;
{
    int j;
    char *spc;

    if (c < ' ' || c > '~') {
	switch (c) {
	  case 0x09:
	      spc = "\\t";
	      break;
	  case 0x0a:
	      spc = "\\n";
	      break;
	  case 0x0d:
	      spc = "\\r";
	      break;
	  case 0xbb: case 0xbc: case 0xbd: case 0xbe: case 0xbf:
	  case 0xc0: case 0xc1: case 0xc2: case 0xc3:
	      fkey[2] = c-0xbb+'1';
	      spc = fkey;
	      break;
	  case 0xc4:
	      spc = "<f10>";
	      break;
	  case 0xe2:
	      spc = "<help>";
	      break;
	  case 0xe1:
	      spc = "<undo>";
	      break;
	  case 0xd2:
	      spc = "<ins>";
	      break;
	  case 0xc7:
	      spc = "<clr>";
	      break;
	  default:
	      hc[1] = hexdig[c>>4];
	      hc[2] = hexdig[c&15];
	      spc = hc;
	}
	conString(spc);
	return strlen(spc);
    }
    else {
	conPut(c);
	return 1;
    }
} /* printChar */



printNum(num)
/* recursive number printing */
unsigned num;
{
    int temp = 0;

    if (num > 9)
	temp = printNum(num/10);
    conPut((num%10)+'0');
    return 1+temp;
} /* printNum */



printString(s)
/* print out a string in human-readable form */
/* returns the number of characters actually printed */
char *s;
{
    int width = 0;

    for(;*s;s++)
	width += printChar((unsigned)(*s));
    return width;
} /* printString */



unsigned
toUpper(c)
register char c;
{
    return (c >= 'a' && c <= 'z')?(c-32):(c);
} /* toUpper */



#define unextend(c)	((c)&0xff)	/* strip sign-extend from a char */


strparse(from,to)
/* parse a string for a macro */
char *from, *to;
{
    unsigned t, c;

    while (*from) {
	c = *from++;
	if (c == '\\' && *from)
	    if (isHex(c = *from++)) {
		t = 0;
		do {
		    if (c >= 'a')
			c -= ('a'-10);
		    else if (c >= 'A')
			c -= ('A'-10);
		    else
			c -= '0';
		    t = (t<<4) | (c & 15);
		} while (*from && isHex(c = *from++));
		c = t;
	    }
	    else
		switch (c) {
		case 'r':
		    c = '\r';
		    break;
		case 'n':
		    c = '\n';
		    break;
		case 't':
		    c = '\t';
		    break;
		}
	*to++ = c;
    }
    *to = 0;
} /* strparse */



addmacro(args)
/* assign a macro to a key/delete a macro */
char *args;
{
    char *key = args;
    unsigned kc;

    args = strchr(args,'=');
    if (args)
	*args++ = 0;
    else			/* point at a null string */
	args = "\0";		/* to avoid ugly address errors */

    strparse(key,key);
    if (strlen(key) != 1) {
	printString(key);
	printString(": improper mapping.");
	println();
	return;
    }
    kc = unextend(*key);
    strparse(args,args);
    if (*args) {			/* define a macro... */
	if (!macro[kc])
	    macro[kc] = malloc(CSIZE);
	if (macro[kc])
	    strcpy(macro[kc],args);
	else {
	    printString("out of memory.");
	    println();
	}
    }
    else if (macro[kc]) {		/* undefine a macro */
	free(macro[kc]);
	macro[kc] = NULL;
    }
} /* addmacro */



addphone(entry)
/* add (or delete) a phonenumber to the phonebook */
char *entry;
{
    int c;

    if (*entry == '-' && strlen(entry) == 2) {
	c = entry[1]-'a';
	if (c < 0 || c >= 26 || !phone[c]) {
	    printChar('\'');printChar(c+'a');printString("' not a phone #");
	    println();
	}
	else {
	    free(phone[c]);
	    phone[c] = NULL;
	}
	return;
    }
    for (c='a'; c <= 'z' && phone[c-'a'];c++)
	;
    if (c <= 'z') {
	if (phone[c-'a'] = malloc(CSIZE))
	    strcpy(phone[c-'a'],entry);
	else {
	    printString("out of memory.");
	    println();
	}
    }
    else {
	printString("too many phone numbers.");
	println();
    }
} /* addphone */



	/* option setting stuff for lookUp(), setoption() */

#define IMAGE	1
#define TIMEOUT	2
#define CONN	3
#define DIAL	4
#define CFG	5
#define PREFIX	6
#define SUFFIX	7
#define VERBOSE	8
#define TERSE	9
#define ESCCHR	10

static
char *options[] = { "image",
		    "timeout",
		    "conn",
		    "dial",
		    "cfg",
		    "prefix",
		    "suffix",
		    "verbose",
		    "terse",		/* terse == !verbose */
		    "escape",
		    NULL };



static int
lookUp(pat)
/* look up a option name */
char *pat;
{
    register int i, plen;

    if (plen = strlen(pat))
	for (i=0; options[i]; i++)
	    if (strncmp(options[i], pat, plen) == 0)
		return 1+i;
    return 0;
} /* lookUp */


setoption(opt)
/* parse a option string.
 * options come in three flavours; opt=val, opt, & noopt.
 */
char *opt;
{
    char *eqp;
    int to, yes;

    if (eqp = strchr(opt,'=')) {
	*(eqp++) = 0;
	switch (lookUp(opt)) {
	case DIAL:
	    startDial = unextend(*eqp);
	    startConn = FALSE;
	    break;
	case CFG:
	    strcpy(cfgfile,eqp);
	    break;
	case TIMEOUT:
	    to = atoi(eqp);
	    if (to < 10 || to > 60) {
		printString("timeout must be 10 to 60.");println();
	    }
	    else timeOut = to;
	    break;
	case PREFIX:
	    strparse(eqp,prefix);
	    break;
	case SUFFIX:
	    strparse(eqp,suffix);
	    break;
	case ESCCHR:
	    strparse(eqp,eqp);
	    if (strlen(eqp) == 1)
		ESCAPE = unextend(*eqp);
	    else {
		printString("funny escape key");
		println();
	    }
	    break;
	case 0:
	    printString("unknown option ");printString(opt);println();
	    break;
	default:
	    printString(opt);printString(" is a boolean option.");println();
	    break;
	}
    }
    else {
	if (*opt == 'n' && *(1+opt) == 'o') {
	    yes = FALSE;
	    opt += 2;
	}
	else
	    yes = TRUE;
	switch (lookUp(opt)) {
	case CONN:
	    startConn = yes;
	    startDial = -1;
	    break;
	case IMAGE:
	    imageMode = yes;
	    break;
	case VERBOSE:
	    verbose = yes;
	    break;
	case TERSE:
	    verbose = !yes;
	    break;
	case 0:
	    printString("unknown option ");printString(opt);println();
	    break;
	default:
	    printString(opt);printString(" is not a boolean option.");println();
	    break;
	}
    }
} /* setoption */



putcfg(name)
/* write the current configuration (macros, phones #'s, etc) to disk */
char *name;
{
    FILE *fp;
    int c,num;
    char temp[CSIZE];
    char *np;
    int spf;
 
    if (fp = fopen(name,"w")) {
	fputs("# Qt ",fp); fputs(VERSION,fp); fputs("\n\n",fp);
	if (!imageMode)
	    fputs("set noimage\n",fp);
	fputs(verbose?"set verbose\n":"set terse\n",fp);

	fputs("set timeout=",fp);
	for ( np = &temp[CSIZE-1], *np = 0, num = timeOut; num; num /= 10)
	    *--np = (num%10)+'0';
	fputs(np,fp); putc('\n',fp);
	
	temp[0] = ESCAPE; temp[1] = 0;
	fputs("set escape=",fp); fputs(printable(temp),fp);   putc('\n',fp);
	fputs("set prefix=",fp); fputs(printable(prefix),fp); putc('\n',fp);
	fputs("set suffix=",fp); fputs(printable(suffix),fp); putc('\n',fp);
	if (startDial >= 0 && startDial < 26) {
	    fputs("set dial=",fp); putc(startDial+'a',fp); putc('\n',fp);
	}
	else if (startConn)
	    fputs("set conn\n",fp);

	spf = TRUE;		/* if we write out any phone#'s, put a */
				/* blank line before them */
	for (c='a';c<='z';c++)
	    if (phone[c-'a']) {
		if (spf) {
		    putc('\n',fp);
		    spf = FALSE;
		}
		fputs("dial ",fp);
		fputs(phone[c-'a'],fp); putc('\n',fp);
	    }
	spf = TRUE;		/* if we write out any macros, put a */
				/* blank line before them */
	for (c=0;c<0x100;c++)
	    if (macro[c]) {
		if (spf) {
		    putc('\n',fp);
		    spf = FALSE;
		}
		strcpy(temp,"map x=");
		temp[4] = c;
		strcat(temp,macro[c]);
		fputs(printable(temp),fp); putc('\n',fp);
	    }
	fclose(fp);
	return TRUE;
    }
    return FALSE;
} /* writecfg */



getcfg(filename)
/* get configuration info from a file */
/* returns TRUE if file found and read in, FALSE otherwise */
char *filename;
{
    FILE *qtrc;
    char cmd[CSIZE];
    char *p;

    if (filename && (qtrc = fopen(filename,"r")) ) {
	while (fgets(cmd,CSIZE,qtrc)) {
	    p = &cmd[strlen(cmd)-1];	/* zot \n at end of line.. */
	    if (*p == '\n')
		*p = 0;
	    if (*cmd && *cmd != '#') {
		if (p = strchr(cmd,' ')) {
		    *p++ = 0;
		}
		else
		    p = "\0";
		if (strcmp("dial",cmd) == 0)
		    addphone(p);
		else if (strcmp("map",cmd) == 0)
		    addmacro(p);
		else if (strcmp("set",cmd) == 0)
		    setoption(p);
		else {
		    printString("unknown command '");
		    printString(cmd);
		    conPut('\'');
		    println();
		}
	    }
	}
	fclose(qtrc);
	return TRUE;
    }
    return FALSE;
} /* getcfg */



initialize(argc,argv)
/* various initialization stuff: read the configuration file, if any.
 *   set up the cursor & the default baud rate.
 */
char **argv;
{
    unsigned c;

    for (c='a'; c<='z'; c++)	/* clean out the phonebook */
	phone[c-'a'] = NULL;
    for (c=0; c<0x100; c++)	/* and the macro table     */
	macro[c] = NULL;

    cursconf(1,0);		/* enable                  */
    cursconf(3,0);		/* & unblinkify the cursor */
    rsconf(baud,0,-1,-1,-1,-1);	/* stay at 1200 baud today */

    while (--argc > 0)		/* get options from the command line */
	setoption(*++argv);

    getcfg(cfgfile);		/* read the config file, if any */

    printString("Quick & dirty terminal v");printString(VERSION);
    if (verbose) {
	printString(" -- Type '");
	if (startConn) {
	    printChar(ESCAPE);printString("' to command.");
	}
	else
	    printString("?' for help.");
    }
    println();

    if (startConn || (startDial >= 0 && dialnum(startDial)) )
	chatterbox();
} /* initialize */



main(argc,argv)
char **argv;
{
    initialize(argc, argv);
    command();
    if (verbose)
	printString("quit");
    println();
} /* main */
