/*
 * qt: a quick and dirty modem program for the Atari ST.
 *
 * xmodem.c: Xmodem file transfer routines (lifted from nt.p, which lifted
 *           them from citadel)
 * revisions:
 *  10-jan-87 orc:	rewritten from p2.
 *  11-jan-87 orc:	timeOut, imageMode put in.
 *  12-jan-87 orc:	fix sign-extend bug in spacket().
 *  21-jan-87 orc:	add verbose flag.
 *  25-jan-87 orc:	xmodem defines are now in xmodem.h
 */

/*
 * the only publics in this file are xferText, checkmode(), recieve(),
 * and xmodem()
 */

#include "qt.h"
#include "xmodem.h"

#define ERRORMAX 10

int xferText;		/* binary or text xfer? */
static int WCSecNum;	/* sector # on upload */
static int WCfd;	/* file being copied to/from */
static int WCerror;	/* any errors happen during xfer */


extern int timeOut;	/* receive timeout */
extern int imageMode;	/* default mode to xfer files in */
extern int verbose;	/* talk talk talk! */


checkmode(ch)
/* checkmode: find out what mode something will be in */
{
    switch (toUpper(ch)) {
      case 'T':
	printString("ext ");
	xferText = TRUE;
	return TRUE;
      case 'B':
	printString("inary ");
	xferText = FALSE;
	return TRUE;
      default:
	xferText = !imageMode;
    }
    return FALSE;
}



receive(seconds)
/************************************************************************/
/*	receive() gets a modem character, or times out ...		*/
/*	Returns:	char on success else ERROR			*/
/************************************************************************/
int seconds;
{
    long start = gemdos(0x2c/*get time-of-day*/);

    while (gemdos(0x2c) < start+seconds)
	if (auxStat())
	    return auxGet();
    return ERROR;
}



static
readFile()
/************************************************************************/
/*	readFile() accepts a file from modem using Ward Christensen's	*/
/*	protocol.  (ie, compatable with xModem, modem7, yam, modem2...) */
/*	Returns:	TRUE on successful transfer, else FALSE 	*/
/************************************************************************/
{
    int i,firstChar,lastSector,thisSector,thisComplement,tries;
    int bp;
    int lastCR;		/* last char was a CR -- for nonimage xfer */
    int toterr, checksum;
    int badSector, writeError;
    char errMess[CSIZE];
    packet sectBuf;

    lastSector = thisSector = 0;
    tries = toterr = bp = 0;

    lastCR = FALSE;
    writeError = FALSE;

    auxFlush();

    printString("Sector #0 (Try=0, Errs=0)");

    auxPut(NAK);			/* Let's get it rolling 	*/
    do {
	errMess[0] = 0;
	/* get synchronized: */
	do {
	    firstChar = receive(10);
	} while (firstChar != SOH && firstChar != EOT && firstChar != ERROR);

	badSector = (firstChar == ERROR);
	if (firstChar == SOH)  {
	    /* found StartOfHeader -- read sector# in: */
	    thisSector     = receive (1);
	    thisComplement = receive (1);	/* 1's comp of thisSector */

	    if ((thisSector + thisComplement) != 0377)  {
		badSector = TRUE;
		strcpy(errMess,"parity error on sector#");
	    }
	    else {
		if (thisSector == (1+lastSector) % 256)  {
		    /* right sector... let's read it in */
		    checksum = 0;
		    for (i = 0; i < SECTSIZE; i++) {
			sectBuf[i] = auxGet();
			if (xferText)
			    sectBuf[i] &= 0x7f;
			checksum = (checksum+sectBuf[i])&0377;
		    }

		    if (checksum != receive (1))  {
			badSector = TRUE;
			strcpy(errMess,"checksum error");
		    }
		    else {
			tries = 0;
			lastSector = thisSector % 256;
			if (!writeError) {
			    auxPut(ACK);
			    writeError = dwrite(WCfd,sectBuf,SECTSIZE) < SECTSIZE;
			}

			conPut('\r');
			printString("Sector #");printNum(thisSector);
			printString(" (Try=");printNum(tries);
			printString(", Errs=");printNum(toterr);
			printString(")  ");
		    }
		}
		else {
		    /* not expected sector... */
		    if (thisSector != lastSector)  {
			badSector = TRUE;
			strcpy(errMess,"VeryBadSector#");
			write(thisSector);
		    }
		    else {
			/* aha -- sender missed an ACK and resent last: */
			while (receive(1) != ERROR)
			    ;   	/* eat it */
			auxPut(ACK);	/* back in synch! */
		    }
		}
	    }/* } of 'if (thisSector + thisComplement == 255'	*/
	}	/* } of 'if (firstChar == SOH)'			*/

	if (badSector)  {
	    if (errMess[0]) {
		printString(errMess);
		println();
	    }
	    ++tries;
	    if (lastSector) 
		toterr++;

	    while (receive (1) != ERROR) 
		;
	    conPut('\r');
	    printString("Sector #");printNum(thisSector);
	    printString(" (Try=");printNum(tries);
	    printString(", Errs=");printNum(toterr);
	    printString(")  ");
	    auxPut(NAK);
	}
    } while (firstChar != EOT && tries < ERRORMAX && !writeError);

    if (firstChar != EOT || tries >= ERRORMAX)  {
	printString(" (abort)");
	println();
	return FALSE;
    }
    else {
	if (writeError)
	    printString(" (write error)");
	else
	    auxPut(ACK);
	println();
	return !writeError;
    }
}



static
getfile()
/* Get a Xmodem file from remote */
{
    char s[CSIZE];
    unsigned c;

    printString("eceive file? ");getString(s,CSIZE-1);
    if (s[0]) {
	dunlink(s);
	if ((WCfd = dcreat(s,0)) < 0) {
	    printString("cannot create ");printString(s);conPut('.');println();
	    return;
	}
	printString("start char? ");
	printChar(c = getKey());
	println();
	auxPut(c);
	readFile();
	dclose(WCfd);
    }
} /* getfile */



static
spacket(p)
/* Xmodem send a packet */
packet p;
{
    int i, j, m;
    int ck;

    for (i = 0; i < ERRORMAX; i++) {
	conPut('\r');
	printString("Sector #");printNum(WCSecNum);printString(" (try ");
	printNum(i);printString(")");
	auxPut(SOH);
	auxPut(WCSecNum);
	auxPut(~WCSecNum);
	ck = 0;
	for (j = 0; j < SECTSIZE; j++) {
	    ck += ((unsigned)p[j]) & 0xFF;	/* wipe sign extend... */
	    auxPut(p[j]);
	}
	auxPut(ck);
	m = receive(60);
	if (m == ACK || m == CAN)
	    break;
    }
    WCSecNum++;
    if (m == ACK)
	return TRUE;
    WCerror = TRUE;
    printString(" (abort)");
    return FALSE;
} /* spacket */



/************************************************************************/
/*	doWC() initializes the system for a WC download 		*/
/************************************************************************/
doWC(mode)
{
    int retry,m;

    if (mode == STARTUP) {	/* Setup globals for coming fun */
	WCSecNum = 1;
	WCerror = FALSE;
	retry = 0;
	while (TRUE)  {
	    m = receive(timeOut);
	    if (m == NAK) 
		return TRUE;
	    if (m == ERROR || m == CAN || ++retry >= ERRORMAX)
		return FALSE;
	}
    }
    else {
	if (WCerror) 
	    return FALSE;
	for (retry = 0; retry < ERRORMAX; retry++) {
	    auxPut(EOT);
	    if (receive(timeOut) == ACK) 
		return TRUE;
	}
	return FALSE;
    }
} /* doWC */



#define PAGESIZE 5120

static
putfile()
/* xmodem send a file */
{
    int idx, rp, endp;
    char c;
    packet pack;
    char page[PAGESIZE];
    char s[CSIZE];

    printString("end file? "); getString(s,CSIZE-1);
    if (!*s) 
	return;
    if ((WCfd = dopen(s,0)) < 0) {
	println();
	printString("cannot open ");printString(s);conPut('.');
	println();
	return;
    }
    printString("start char? ");	/* away we go... */
    printChar(c = getKey());
    auxPut(c);
    println();

    rp = 0;
    if (doWC(STARTUP))  {
	while ((endp=dread(WCfd,page,PAGESIZE)) > 0)  {
	    for (idx = 0;idx < endp; idx++) {
		pack[rp] = page[idx];
		if (xferText)
		    pack[rp] &= 0x7f;
		rp = (1+rp)%SECTSIZE;
		if (rp == 0 && !spacket(pack)) 
		    goto fail;
	    }
	}

	if (rp > 0)  {
	    while (rp < SECTSIZE)
		pack[rp++] = 'Z'-'@';
	    if (!spacket(pack))
		goto fail;
	}
	if (!doWC(STARTUP+1)) 
	    printString("(timeout)");
    }
    else
	printString("(timeout)");
  fail:
    println();
    dclose(WCfd);
} /* putfile */



xmodem()
/* Xmodem file transfer menu */
{
    unsigned ch;

    printString("xmodem ");
    
    printChar(ch = getKey());
    if (checkmode(ch))
	printChar(ch = getKey());
    switch (toUpper(ch)) {
    case 'R':
	getfile();
	break;
    case 'S':
	putfile();
	break;
    case '?':
	println();
	printString("x[m]r - Get a file using Xmodem protocol"); println();
	printString("x[m]s - Send a file ..."); println();
	printString(" [m]  - transfer mode; b(inary or t(ext"); println();
	printString("x?    - Print this message"); println();
	break;
    default:
	conString("?\007");
	if (verbose)
	    printString(" -- type 'x?' for help.");
    case ESC:
	println();
    }
} /* xmodem */
