/*
 * Main Module
 *
 *
 *	ST users @ Case Western Reserve University
 */

#include <osbind.h>
#include <setjmp.h>
#include "xmdm.h"

#define	esc	27
#define cr	0x0d

 	/* Global variable  definitions, extern everywhere else */
char bufr[ BUFSIZ ];		/* File buffer */
char scratch[SCRATCH];  	/* Scratch buffer */
char capture = 0;		/* Ascii capture flag   */
unsigned int buf_index = 0;	/* Index to file buffer */
int  currb;			/* Current Baud Rate    */
int dchanged = 0;		/* Directory has been updated??      */
long *ms_ptr;			/* Pointer to my screen memory, aligned at
				   a 256 bytes boundary */
long m_screen[8*1024+32];	/* My screen memory
				   32K bytes + 256 Bytes guard for alignement
				   In the worst case when we align we have to
				   go 255 bytes from &m_screen[0], hence the
				   256 Byte guard is required */

jmp_buf abort_env,	  /* Long jump when transfers are aborted */
        time_env;	  /* Long jump on timeout		  */

DTA_T my_dta;		  /* Disk Transfer address for Find first etc */

	
#ifdef DEBUG
int debug  = 1;			/* Debug flag */
#endif

 	/* Types */
/* The structure returned by Iorec(), really ptr to this type */
typedef struct {
	char *ibuf;
	int ibufsiz;
	int ibufhd;
	int ibuftl;
	int ibuflow;
	int ibufhi;
} IOREC;

/* The type word */
typedef short word;


 	/* Globals belonging to this module only */
IOREC save,	/* the original Iorec is saved here for the duration of this
		   process */
 	*p;	/* ptr returned by Iorec() */

char iobuf[IBUFSIZ]; /* My large Rs232 receive buffer */

word	rs232 = 1,		/* Ports */
        console = 2;
word	speed,			  /* rs232 setup parameters */
 	flowctl = 1,
	ucr = -1,
	rsr = -1,
	tsr = -1,
	scr = -1;

/*
 * SetIoBuf() - Save the systems Rs232 buffer and install my large
 * Rs232 buffer.
 *
 */
void SetIoBuf()
{
	/* Get pointer to Rs232 input record */
	p = Iorec(0);
	
	/* Save the info */
	save.ibuf	= p->ibuf;
	save.ibufsiz	= p->ibufsiz;
	save.ibufhd	= p->ibufhd;
	save.ibuftl	= p->ibuftl;
	save.ibuflow	= p->ibuflow;
	save.ibufhi	= p->ibufhi;
	
	/* Install my buffer in its place */
	p->ibuf		= &iobuf[0];
	p->ibufsiz	= IBUFSIZ;
	p->ibuflow	= IBUFSIZ/4;
	p->ibufhi	= IBUFSIZ / 4 * 3;
	p->ibufhd = p->ibuftl = 0;
	
}

/*
 * ResetIoBuf() - Reset the Rs232 buffer to the saved (system's) buffer
 *
 */
void ResetIoBuf()
{
	p->ibuf		= save.ibuf;
	p->ibufsiz	= save.ibufsiz;
	p->ibuflow	= save.ibuflow;
	p->ibufhi	= save.ibufhi;
	p->ibufhd	= save.ibufhd;
	p->ibuftl	= save.ibuftl;
}

/* the following code is stolen from Tsu Wang - with permission of course */

/*
 * setRs232() - set rs232 port configuration
 */
void setRs232 ()
{
	char ch;
	long conin;
	
	Cconws("Baud rate: ");
	EscSeq(p);
	Cconws("0=19200 1=9600, 2=4800, 3=2400, 4=1200, 5=300\r\n");
	EscSeq(q);
	Cconout('\t');
	EscSeq(p);
	Cconws("What speed==>");
	EscSeq(q);
	
	conin = Bconin(console);	/* get speed */
	if (conin == 0x610000)
	{
		/* He hit <UNDO> */
		if(capture)
		{
			Cconws("Closing Capture File\r\n");
			c_capture();
		}
		his_screen();
		ResetIoBuf();
		finish();
	}
	ch = (char) (conin & 0x7f);
	Cconout(' ');
	switch (ch)
	{
	    case '0':		/* 19200 */
		speed = 0;
		Cconws("19200");
		break;
		
	    case '1':
		speed = 1;	/* 9600 */
		Cconws("9600");
		break;
		
	    case '2':
		speed = 2;	/* 4800 */
		Cconws("4800");
		break;
		
	    case '3':
		speed = 4;	/* 2400 */
		Cconws("2400");
		break;
		
	    case '4':
		speed = 7;	/* 1200 */
		Cconws("1200");
		break;
		
	    case '5':
		speed = 9;	/* 300 */
		Cconws("300");
		break;
		
	    default:
		speed = 1;	/* 9600 */
		Cconws("1200");
	}
	Cconws(" Baud\r\n");
	
	/* Set new Baud rate */
	currb = speed;
	Rsconf(speed, flowctl, ucr, rsr, tsr, scr);

	/* get rid of extraneous characters that get sent out */
	Bconout(rs232,(word)'\b');
	Bconout(rs232,(word)'\b');
	
}

/*
 * help() - display help info and menu
 */
void help ()
{
	word c;
	long conin;
	char *r_filename();
	
	EscSeq(j);		/* Save Cursor */
	my_screen();		/* Switch to my screen memory */
	EscSeq(v);		/* wrap at end of line */
	EscSeq(E);		/* clear screen */
	
	/* Put up menu */
	Cconws("\r\n\t");
	EscSeq(p);		/* reverse video */
	Cconws("Undo");
	EscSeq(q);		/* quit reverse video */
	Cconws(" to exit.\r\n");
	
	Cconws("\t");
	EscSeq(p);		/* reverse video */
	Cconws("Help");
	EscSeq(q);		/* quit reverse video */
	Cconws(" for this message.\r\n");
	
	Cconws("\t");
	EscSeq(p);		/* reverse video */
	Cconws("C or c");
	EscSeq(q);		/* quit reverse video */
	Cconws(" to toggle Capture.\r\n");
	
	Cconws("\t");
	EscSeq(p);		/* reverse video */
	Cconws("S or s");
	EscSeq(q);		/* quit reverse video */
	Cconws(" to Send a file using Xmodem.\r\n");
	
	
	Cconws("\t");
	EscSeq(p);		/* reverse video */
	Cconws("R or r");
	EscSeq(q);		/* quit reverse video */
	Cconws(" to Receive a file using Xmodem.\r\n");

	Cconws("\t");
	EscSeq(p);		/* reverse video */
	Cconws("O or o");
	EscSeq(q);		/* quit reverse video */
	Cconws(" for OS services.\r\n");
	
	Cconws("\t");
	EscSeq(p);		/* reverse video */
	Cconws("Return");
	EscSeq(q);		/* quit reverse video */
	Cconws(" to do nothing.\r\n");
	
	Cconws("\t");
	EscSeq(p);		/* reverse video */
	Cconws("B or b");
	EscSeq(q);		/* quit reverse video */
	Cconws(" to set baud rate.     Default is ");
	EscSeq(p);
	Cconws("9600 Baud.\r\n\r\n");
	EscSeq(q);

	/* get response */
	conin = Bconin(console);

	if (conin == 0x610000)
	{
		/* He hit <UNDO> */
		if(capture)
		{
			Cconws("Closing Capture File\r\n");
			c_capture();
		}
		his_screen();
		ResetIoBuf();
		finish();
	}
	
	switch((int)(conin & 0x7f))
	{
	    case 'B':
	    case 'b':
		/* Set baud rate */
		setRs232();
		break;
		
	    case 'C':
	    case 'c':
		/* Toggle Capture */
		if(capture)
		    c_capture();
		else
		    s_capture();
		break;
		
	    case 'S':
	    case 's':
	    {
		    /* Send a file using Xmodem */
		    register char *f;
		    if(capture)
		    {
			    /* We are capturing */
			    Cconws("Closing Capture File\r\n");
			    c_capture();
		    }

		    if((f = r_filename()) == (char *)NULL)
		    {
			    /* user cancelled transfer */
			    canit();
			    flushinput();
			    break;
		    }

		    /* Set no flow Control */
		    Rsconf(-1,0,-1,-1,-1,-1);

		    /* Send file */
		    if(sendfile(f) == FALSE)
			Cconws("\r\nTransfer not Completed\r\n");
		    else
			Cconws("\r\nTransfer Complete\r\n");
		    
		    /* Flow Control On */
		    Rsconf(-1,1,-1,-1,-1,-1);
		    Bconout(rs232,(word)'\b');
		    Bconout(rs232,(word)'\b');
	    }
		break;
		
	    case 'r':
	    case 'R':
	    {
		    /* receive using Xmodem */
		    register char *f;
		    if(capture)
		    {
			    /* We are capturing */
			    Cconws("Closing Capture File\r\n");
			    c_capture();
		    }

		    if((f = r_filename()) == (char *)NULL)
		    {
			    /* user cancelled transfer */
			    canit();
			    flushinput();
			    break;
		    }
		    

		    /* No flow control */
		    Rsconf(-1,0,-1,-1,-1,-1);

		    /* receive file */
		    if(readfile(f) == FALSE)
			Cconws("\r\nTransfer not Completed\r\n");
		    else
			Cconws("\r\nTransfer Complete\r\n");
		    
		    /* flow control on */
		    Rsconf(-1,1,-1,-1,-1,-1);
		    Bconout(rs232,(word)'\b');
		    Bconout(rs232,(word)'\b');
	    }
		break;
	  
	    case 'o':
	    case 'O':
		/* Os Services */
		os();
		return;
		
	    default:
		Cconws("No Change\r\n");
	}

	/* Wait for a key hit */
	hit_key();
	/* back to terminal screen */
	his_screen();
}

main ()
{
	word	c;		/* rs232 input */
	long	conin;
	int	i;
	
	/* Set up Dta */
	Fsetdta(&my_dta);

	ms_ptr = (long *) ((0xffffff00 & ((long)(&m_screen[0]))) + 0x00000100);
	EscSeq(e);		/* Turn on the cursor */
	EscSeq(v);		/* wrap at end of line */
	EscSeq(E);		/* clear screen */
	SetIoBuf();

	Rsconf((word) 1, flowctl, ucr, rsr, tsr, scr);  /* init set */
	Bconout(rs232,(word)'\b');
	Bconout(rs232,(word)'\b');
	currb = 1;

	help();

	while (0 == 0)	/* infinite loop */
	{
		i = 0;
		while (Bconstat(rs232) == 0xffff)
		{
			/* Char at Modem */
			c = Bconin(rs232) & 0x7f;
			Bconout(console, c);
			if(capture)
			    /* We are capturing */
			    a_capture(c);

			/* Check the console once in a while */
			/* important at High speeds */
			if ((++i) & 32)
			    if (Bconstat(console) == 0xffff)
	      		    {
				    conin = Bconin(console);
				    Bconout(rs232, (word) (conin & 0x7f));
				    i = 0;
			    }
		}
		
		if (Bconstat(console) == 0xffff)
		{
			/* Char at Console */
			conin = Bconin(console);
			if (conin == 0x610000)		/* Undo */
			{
				if(capture)
				{
					Cconws("Closing Capture File\r\n");
					c_capture();
				}
				ResetIoBuf();
				finish();
			}
			
			if (conin == 0x620000)		/* Help */
			    help();
			else
			    Bconout(rs232, (word) (conin & 0x7f));
		}
	}
}

finish()
{
	Pterm0();
}
