/*
 * Operating System Services Module
 *
 *
 *	ST users @ Case Western Reserve University
 */
#include <osbind.h>
#include <ctype.h>
#include "xmdm.h"

#define clear_screen()  Cconws("\033H\033J")
#define inv()		EscSeq(p)
#define normal()	EscSeq(q)
#define mvto(r,c)	EscSeq(Y);Cconout(r+040);Cconout(c+040)
#define ceol()		EscSeq(K)
#define show_cursor()	EscSeq(e)
#define hide_cursor()	EscSeq(f)

	/* External Variables */
extern char capture;		/* capture flag */

	/* Globals */
static char _cwd[68];		/* Current working directory */
static long _fs = -1L;		/* Free space		     */


/*
 * Show the current working directory
 *
 */
ls()
{
	register DTA_T *dta;
	int row, col;
	char path[4];

	row = col = 0;
	dta = Fgetdta();
	strcpy(path,"*.*");

	if(Fsfirst(path, (0x01 | 0x10 | 0x20)) != 0)
	{
		Cconws("\t");
		Cconws(_cwd);
		Cconws(": has no entries\r\n");
		hit_key();
		return;
	}
	
	hide_cursor();
	clear_screen();

	mvto(row,col);
	Cconws(dta->filename);
	inc(&row,&col);

	while(Fsnext() == 0)
	{
		mvto(row,col);
		Cconws(dta->filename);
		inc(&row,&col);
	}
	show_cursor();
	hit_key();
}

/*
 * inc row and col
 *
 */
inc(r,c)
register int *r;
register int *c;
{
	*c = *c + 20;
	if(*c > 60)
	{
		*c = 0;
		*r = *r + 1;
		if(*r == 24)
		{
			*r = 0;
			mvto(25,0);
			hit_key();
			clear_screen();
		}
	}
}

/*
 * Change Current Working Directory
 *
 */
cd()
{
	register char *f, *path, *d;
	register int drive;
	extern char *readline(), *index();

	Cconws("Directory or <RETURN> to Cancel: ");
	if((f = readline()) == (char *)NULL)
	{
		hit_key();
		return;
	}
	
	if((d = index(f,':')) == (char *)NULL)
	{
		/* Drive was not specified, must mean the current drive */
		path = f;
	}
	else
	{
		d--;
		if(isupper(*d))
		{
			*d = tolower(*d);
		}
		drive = *d - 'a';
		path = (char *)((long)d + 2L);

		/* Set the Drive */
		if(Dsetdrv(drive) < 0)
		{
			Cconws("Could not set drive to ");
			Cconout(*d);
			Cconout(':');
			Cconws("\r\n");
			hit_key();
			return;
		}

	}


	/* Set the Path */
	if(Dsetpath(path) != 0)
	{
		Cconws("Could not set current directory to ");
		Cconws(f);
		Cconws("\r\n");
		hit_key();
		return;
	}
	fs();
	cwd();
	
}

/*
 * Cwd - figure out the current working directory
 *
 */
cwd()
{
	register int drive;
	
	drive = Dgetdrv();
	Dgetpath(&_cwd[3],0);
	_cwd[0] = drive + 'a';
	_cwd[1] = ':';
	_cwd[2] = (_cwd[3] == '\\')? ' ' : '\\'; /* Kludge city */

}


/*
 * Delete a file
 *
 */
rm()
{
	register char *f;
	extern char *readline();

	Cconws("Filename to delete or <RETURN> to Cancel: ");
	if((f = readline()) == (char *)NULL)
	{
		hit_key();
		return;
	}

	if(Fdelete(f) != 0)
	{
		Cconws("Could not delete ");
		Cconws(f);
		Cconws("\r\n");
		hit_key();
		return;
	}
	fs();
	
}


/*
 * fs - get free space
 *
 */
fs()
{
	struct {
	    long b_free;
	    long b_total;
	    long b_secsiz;
	    long b_clsiz;
	} fbuf;

	Dfree(&fbuf,0);
	_fs = fbuf.b_free * fbuf.b_clsiz * fbuf.b_secsiz;
}

/*
 * Os - top level function for Os Services
 *
 */
os()
{
	register long conin;

	if(_fs < 0)
	{
		cwd();
		fs();
	}

	while(1)
	{
		clear_screen();
		mvto(2,35);
		inv();
		Cconws("OS  Services");
		normal();

		mvto(5,1);
		
		Cconws("\tThe Current Working Directory is \"");
		Cconws(_cwd);
		Cconws("\"\r\n\tThe Current Drive has ");
		prinl(_fs);
		Cconws(" Bytes [");
		prinl((long)(_fs/1024L));
		Cconws("K] Free.\r\n\n");
		
		
		/* Put up menu */
		Cconws("\r\n\t");
		EscSeq(p);		/* reverse video */
		Cconws("Undo");
		EscSeq(q);		/* quit reverse video */
		Cconws(" to exit the emulator.\r\n");
		
		Cconws("\t");
		EscSeq(p);		/* reverse video */
		Cconws("l");
		EscSeq(q);		/* quit reverse video */
		Cconws(" to list directory.\r\n");
		
		Cconws("\t");
		EscSeq(p);		/* reverse video */
		Cconws("d");
		EscSeq(q);		/* quit reverse video */
		Cconws(" to delete a file.\r\n");
		
		Cconws("\t");
		EscSeq(p);		/* reverse video */
		Cconws("c");
		EscSeq(q);		/* quit reverse video */
		Cconws(" to change the current working directory.\r\n");
		
		Cconws("\t");
		EscSeq(p);		/* reverse video */
		Cconws("Return");
		EscSeq(q);		/* quit reverse video */
		Cconws(" to return to the emulator.\r\n\n\n\n");
		
		/* get response */
		conin = Bconin(2);
		
		if (conin == 0x610000)
		{
			/* He hit <UNDO> */
			if(capture)
			{
				Cconws("Closing Capture File\r\n");
				c_capture();
			}
			his_screen();
			ResetIoBuf();
			finish();
		}
		
		switch((int)(conin & 0x7f))
		{
		    case 'd':
			/* Delete a file */
			rm();
			break;
			
		    case 'c':
			/* Cd */
			cd();
			break;
			
		    case 'l':
			/* Ls */
			ls();
			break;
			
			
		    case '\r':
			his_screen();
			return;
			
		    default:
		}
		
	}
	
}
