/* 
 * COMM, a tiny terminal, by Steve Yelvington.
 * This program was written for Mark Johnson's
 * shareware C compiler. 
 */

#include <mybind.h> /* a file with some bios and xbios bindings */

#define AUX 1
#define CON 2

#define BAUD 7 /* this means 1200 */

/* 
 * To recompile COMM for other baud rates, change the BAUD define
 * to one of these values:
 * 0 for 19200
 * 1 for 9600
 * 2 for 4800
 * 3 for 3600
 * 4 for 2400
 * 5 for 2000
 * 6 for 1800
 * 7 for 1200
 * 8 for 600
 * 9 for 300
 */

main()
{
	register c = 0x00;

/* setting up... */

	Cursconf(1,0); /* force cursor on */
	Cursconf(3,0); /* blink no more! */

/* in the following, 033 = escape and V turns wrap on */
	Cconws("COMM is active. Control-d to exit.\n\r\033v\0"); 
	
 /* BAUD bps, no flow control */
	Rsconf(BAUD,0,-1,-1,-1,-1);

/* entering terminal mode */
				
	while (c != 0x04)   /* ctrl-d will terminate comm! */
		{
		if(Bconstat(AUX)) /* has modem sent a byte? */
			{
			c = Bconin(AUX); 
			
			/* strip hi bit -- overflow is ignored */
			if(c>0x7f) c+=0x80; 
			
			Bconout(CON, c);
			c = 0x00; /* prevents remote from killing COMM */
			}
		if(Bconstat(CON)) /* has key been pressed? */
			{
			c=Bconin(CON);
			Bconout(AUX, c);
			}
		}

/*
 * Note: The cursor is NOT turned off. A GEM program will blank
 * it, so it's no big deal. If you turn it off here, you won't
 * have one when you return to microEMACS or a command-line
 * processor.
 */
	exit(0);
}
