/* bmodem - the BIX modem program */
/*
	by David Betz, BYTE Magazine/BIX
*/

#include <stdio.h>
#include <ctype.h>
#include <setjmp.h>
#include <osbind.h>

/* set the stack size */
long _stksize = 10000;

/* useful definitions */
#define TRUE	1
#define FALSE	0

/* manifest constants */
#define ENVSIZE	2048

/* program banner */
#define BANNER \
"BMODEM v2.3a by David Betz -- type the HELP key for more information\n"

/* disk transfer area structure */
typedef struct {
    unsigned char dta_dmy[21];	/* reserved for gemdos */
    unsigned char dta_attr;	/* file attributes */
    unsigned short dta_time;	/* creation time */
    unsigned short dta_date;	/* creation date */
    long dta_size;		/* file size */
    char dta_name[14];		/* file name */
} DTA;

/* global variables */
int crc;		/* crc mode flag */
int big;		/* big block mode flag */
int esc;		/* escape character */
int half;		/* half duplex */
FILE *fp;		/* open file pointer */
FILE *cfp;		/* capture file pointer */
jmp_buf xfer_error;	/* escape trap */

/* external routines */
extern FILE *fopen();

/* forward declarations */
long filesize();

/* main - the main routine */
main(argc,argv,envp)
  int argc; char *argv[],*envp[];
{
    int i;

    /* initialize */
    initenv(envp,ENVSIZE);	/* initialize the environment routines */
    big = FALSE;		/* default to xmodem */
    crc = FALSE;		/* default to checksum */
    half = TRUE;		/* default to half duplex */
    esc = 0x22B;		/* default escape character to ^\ */
    cfp = NULL;			/* no capture file yet */

    /* initialize the external routines */
    md_init(1200);	/* initialize the modem port at 1200 baud */
    te_init();		/* initialize the terminal emulator */

    /* print the banner line */
    printf(BANNER);

    /* read the initialization file */
    script("bmodem.ini");

    /* execute any script files that were mentioned on the command line */
    for (i = 1; i < argc; ++i)
	if (!script(argv[i])) {
	    error("can't open script file: %s",argv[i]);
	    exit(1);
	}

    /* enter terminal mode */
    while (mdgetc(TRUE) >= 0)
	;

    /* finish up and exit */
    cleanup();
}

/* cleanup - finish up and exit */
cleanup()
{
    /* close the capture file */
    if (cfp) fclose(cfp);

    /* finish up with the modem port */
    md_done();

    /* exit successfully */
    exit(0);
}

/* putch - put a modem character into the current file */
putch(ch)
  int ch;
{
    putc(ch,fp);
}

/* getfile - receive a file */
int getfile(fname)
  char *fname;
{
    int sts;

    /* open the output file */
    if ((fp = fopen(fname,"wb")) == NULL)
	return (error("can't open for output: %s",fname));

    /* turn off xon/xoff */
    Rsconf(-1,0,-1,-1,-1,-1);

    /* transfer the data */
    sts = (setjmp(xfer_error) ? FALSE : dl_rcv(putch,crc));

    /* report errors */
    if (sts)
	printf("\007\n[ transfer complete ]\n");
    else
	printf("\007\n[ transfer failed ]\n");

    /* turn on xon/xoff */
    Rsconf(-1,1,-1,-1,-1,-1);

    /* close the file */
    fclose(fp);

    /* return status */
    return (sts);
}

/* getch - get a character from the current file */
int getch()
{
    return ((int)getc(fp));
}

/* putfile - send a file */
int putfile(fname)
  char *fname;
{
    long fsize,bsize;
    int sts;

    /* get the file size */
    if ((fsize = filesize(fname)) < 0L)
	return (error("can't access: %s",fname));

    /* open the input file */
    if ((fp = fopen(fname,"rb")) == NULL)
	return (error("can't open for input: %s",fname));

    /* turn off xon/xoff */
    Rsconf(-1,0,-1,-1,-1,-1);

    /* transfer the data */
    bsize = (big ? 1024L : 128L);
    printf("[ %s: %ld bytes, %ld blocks ]\n",fname,
           fsize,(fsize + bsize - 1L) / bsize);
    sts = (setjmp(xfer_error) ? FALSE : dl_snd(getch,crc,big));

    /* report errors */
    if (sts)
	printf("\007\n[ transfer complete ]\n");
    else
	printf("\007\n[ transfer failed ]\n");

    /* turn on xon/xoff */
    Rsconf(-1,1,-1,-1,-1,-1);

    /* close the file */
    fclose(fp);

    /* return status */
    return (sts);
}

/* filesize - get the size of a file */
long filesize(fname)
  char *fname;
{
    DTA dta,*olddta;
    olddta = (DTA *)Fgetdta(); Fsetdta(&dta);
    if (Fsfirst(fname,0)) {
	Fsetdta(olddta);
	return (-1L);
    }
    Fsetdta(olddta);
    return (dta.dta_size);
}
