/* cmcmenv.c		A module of cm test program */

/*	860612	Mark Mallett		*/

/* This file has routines to handle, and process commands dealing with,
   [environment] variables.

	envstr		Return addr of environment area in form
			  suitable to pass to Pexec
	initenv		Init variables
	insenv		Insert or replace environment variable
	remenv		Remove environment variable
*/

#include <stdio.h>
#define NUL	'\0'

/* External routines */

extern	char	*calloc();
extern	char	*getenv();

/* External data */

extern	char	**environ;		/* Environment table */

/* Local routines */


/* Local data */

static	int	Eafree;			/* # bytes free in env area */
static	char	*Earea;			/* Environment area */
static	int	Esbase;			/* Offset to first string in Earea */
static	int	Esize;			/* Size of environment area */

/*

*//* insenv(name, value)

	Insert a name/value pair into the environment


Accepts :

	name		Name to store
	value		Value of environment variable

Returns :

*/

insenv(name, value)

	char		*name;		/* Environment variable name */
	char		*value;		/* Value string */

{
	char		*sptr;		/* String pointer */
	int		i;		/* Index */
	int		n;		/* Scratch */

if ((sptr = getenv(name)) != NULL)	/* Already there? */
    {
    n = strlen(value)-strlen(sptr);	/* Difference in lengths */
    if (n > Eafree)			/* If not enuf room */
	{
	printf("Not enough environment space.\n");
	return;
	}
    eadjust(sptr, n);			/* Adjust environment area */
    strcpy(sptr-n, value);		/* Copy new value */
    return;
    }

/* Get length of new string space needed */
n = strlen(name) + strlen(value) + 2;	/* Plus "=" and NUL */
if ((n + sizeof(char *)) > Eafree)	/* see if space (plus pointer entry) */
    {
    printf("Not enough environment space.\n");
    return;
    }

/* Adjust environment strings. */
eadjust(&Earea[Esize], n);

sptr = &Earea[Esize-n];			/* Where to put new string */

strcpy(sptr, name);			/* Build the environment entry */
strcat(sptr, "=");
strcat(sptr, value);

/* Find the spot to put the new string */
for (i = 0; environ[i] != NULL; i++)
    if (encmp(name, environ[i]) < 0)
	break;

/* Shuffle up pointers */
for (n = i; environ[n] != NULL; n++)	/* First find end of table */
    ;
do
    environ[n+1] = environ[n];		/* Copy pointers */
while (n-- != i);

environ[i] = sptr;			/* Add pointer to table */
Eafree = Eafree - sizeof(char *);	/* Account for the new table entry */
}
/*

*//* remenv(name)

	Remove an environment entry

Accepts :

	name		Name of environment variable to remove

Returns :

	<name not in environment>

*/

remenv(name)

	char		*name;		/* Name to remove */

{
	int		i;		/* Scratch */
	char		*eptr;		/* entry pointer */

if ((eptr = getenv(name)) != NULL)	/* If it is in the table */
    {
    eptr = eptr - (strlen(name) + 1);	/* Get entry address */
    for (i = 0; environ[i] != NULL; i++)
	if (environ[i] == eptr)		/* If found entry */
	    {
	    do				/* Shuffle down the list */
	        environ[i] = environ[i+1];
	    while (environ[++i] != NULL);
	    Eafree = Eafree+sizeof(char *);  /* Account for 1 less ptr */

	    eadjust(eptr, -(sizeof(eptr)+1));   /* Shuffle string area */
	    break;
	    }
    }
}
/*

*//* eadjust(ptr, nbytes)

	Adjust string area @ptr by nbytes

Accepts :

	ptr		Addr of where to start adjustment
	nbytes		Number of bytes to adjust
			if positive, strings previous to ptr will be
			   moved backwards (i.e., @ptr will grow)
			if negative, strings not before ptr will be
			   moved forwards (i.e., @ptr will shrink)

Returns :

	Eafree		Adjusted
	Esbase		Adjusted
	environ		pointers adjusted as necessary

*/

eadjust(ptr, nbytes)

	char		*ptr;		/* Addr of string to adjust */
	int		nbytes;		/* Number of bytes */

{
	char		*sptr;		/* Source pointer */
	char		*dptr;		/* Destination pointer */
	int		i,n;		/* Scratch */

sptr = &Earea[Esbase];			/* Start of string area */
dptr = sptr - nbytes;			/* Where it will move to */
n = ptr - sptr;				/* Number of bytes to move */

if (nbytes > 0)				/* If moving backwards */
    while (n--)				/* Move the bytes */
	*dptr++ = *sptr++;

if (nbytes < 0)				/* If moving forwards */
    {
    sptr += n;				/* Start from end of strings */
    dptr += n;
    while (n--)				/* Move the data */
	*--dptr = *--sptr;
    }

/* Adjust pointers to any strings that have just been moved */
for (i = 0; environ[i] != NULL; i++)
    if (environ[i] < ptr)
	environ[i] -= nbytes;

/* Adjust base and free */
Esbase -= nbytes;
Eafree -= nbytes;
}
/*

*//* initenv(envp)

	Initialize variables stuff


Accepts :

	envp		environment var table pointer as given to main()
	envsiz		Size of environment area to allocate

Returns :

	< local environment stuff set up >

*/

initenv(envp, envsiz)

	char		**envp;		/* Addr of env var table */
	int		envsiz;		/* Size of env area */

{
register int		i;		/* Scratch */
	int		n;		/* Scratch */
	char		*sptr;		/* String pointer */
	char		name[50];	/* Var name */

/* Determine a minimum size needed to contain all the environment
   strings as well as the table of pointers to those strings, plus
   some amount of extra for expansion. */

n = 0;					/* #bytes needed for strings */
for (i = 0; envp[i] != NULL; i++)	/* Count vars in passed list */
    n += (strlen(envp[i]) + 1);		/* Count chars */

Esize = n + 200 + (i*sizeof(char *));	/* Minimum size to allocate */
if (Esize < envsiz)			/* If requested more than this */
    Esize = envsiz;			/*   then use requested amount   */

if ((Earea = calloc(1, Esize+1)) == NULL)
    {
    printf("FATAL: Can not init environment area.\n");
    exit(1);
    }

environ = (char **)Earea;		/* Table is at start of area */
Esbase = Esize;				/* Offset to first string */
Eafree = Esize;				/* Room left in env area */

/* Add all the environment variables via insenv() so that they are */
/* properly sorted */

for (i = 0; envp[i] != NULL; i++)	/* Fill in the table */
    {
    sptr = envp[i];
    for (n = 0; *sptr != '='; )		/* Get name w/o equals */
	{
	if (*sptr == NUL)		/* handle case with no equals */
	    {
	    --sptr;
	    break;
	    }
	name[n++] = *sptr++;
	}
    name[n] = NUL;
    insenv(&name[0], sptr+1);		/* Insert new var */
    }
}
/*

*//* envstr()

	Return environment strings in form to pass to a program.

*/

char *envstr()

{
return(&Earea[Esbase]);
}
/*

*//* encmp(str1, str2)

	Compare two environment string names

Like strcmp(), only strings may terminate with '=' as well as NUL.

*/

int encmp(str1, str2)

	char		*str1, *str2;

{
	char		c1, c2;

for (;;)
    {
    c1 = *str1++;
    c2 = *str2++;
    if (c1 == '=')
	c1 = NUL;
    if (c2 == '=')
	c2 = NUL;
    if (c1 < c2)
	return(-1);
    else if (c1 > c2)
	return(1);
    else if (c1 == NUL)
	return(0);
    }
}
