/* mdio - modem i/o routines for bmodem (Atari-ST version) */
/*
	by David Betz, BYTE Magazine/BIX
*/

#include <stdio.h>
#include <setjmp.h>
#include <osbind.h>

/* routine status codes */
#define DT_TIME	-1

/* one second timeout constant */
#define ONESEC	5000

/* baud rate table */
static struct {
    int rate;
    int code;
} rates[] = {
	19200,	0,
	9600,	1,
	4800,	2,
	3600,	3,
	2400,	4,
	2000,	5,
	1800,	6,
	1200,	7,
	600,	8,
	300,	9,
	200,	10,
	150,	11,
	134,	12,
	110,	13,
	75,	14,
	50,	15,
	-1
};

/* shift status info */
#define CONTERM	((char *)0x484)
#define SHFT	0x08

/* external variables */
extern jmp_buf xfer_error;

/* md_init - initialize the modem port */
int md_init(rate)
  int rate;
{
    long oldsp;
    int i;

    /* lookup the baud rate */
    if (rate) {
	for (i = 0; rates[i].rate >= 0; ++i)
	    if (rates[i].rate == rate) {
		Rsconf(rates[i].code,0,-1,-1,-1,-1);
		while (Bconstat(1) < 0)
		    Bconin(1);
		goto got_rate;
	    }
	printf("Unknown baud rate: %d\n",rate);
	return (0);
    }

got_rate:
    /* enable shift status */
    oldsp = Super(0L);
    *CONTERM |= SHFT;
    Super(oldsp);

    /* return successfully */
    return (1);
}

/* md_done - terminate modem communications */
md_done()
{
    long oldsp;

    /* disable shift status */
    oldsp = Super(0L);
    *CONTERM &= ~SHFT;
    Super(oldsp);
}

/* md_get - get a character from the modem with timeout */
int md_get(tmo)
  int tmo;
{
    int i,j;

    for (i = 0; i < tmo; ++i)
	for (j = 0; j < ONESEC; ++j)
	    if (Bconstat(1) < 0)
		return (Bconin(1));
	    else if (Bconstat(2) < 0 && (Bconin(2) & 0xFF) == '\033')
		longjmp(xfer_error,1);
    return (DT_TIME);
}

/* md_flush - flush the modem input buffer */
md_flush()
{
    while (Bconstat(1) < 0)
	Bconin(1);
}

/* md_put - output a character to the modem */
md_put(ch)
  int ch;
{
    Bconout(1,ch);
}

/* md_write - write a packet to the modem */
md_write(buf,len)
  char *buf;
  int len;
{
    while (--len >= 0)
	md_put(*buf++);
}
