/* script.c - bmodem script handler */
/*
	by David Betz, BYTE Magazine/BIX
*/

#include <stdio.h>
#include <ctype.h>
#include <osbind.h>

/* useful definitions */
#define TRUE	1
#define FALSE	0

/* cycles per second */
#define HZ	60

/* program limits */
#define LSIZE	200	/* maximum line size */
#define NSIZE	100	/* size of modem data buffer for EXPECT */
#define CSIZE	8192	/* capture buffer size */

/* default environment variable values */
#define DEFPATH		".bin,,\\bin"
#define DEFSUFF		".prg,.tos,.ttp"
#define DEFBMODEM	",\\bmodem"

/* command interpreter status codes */
#define CS_OK		1
#define CS_FLUSH	2
#define CS_ABORT	3

/* lookup status codes */
#define LS_FOUND	1
#define LS_UNKNOWN	2
#define LS_AMBIGUOUS	3

/* lookup symbol table structure */
typedef struct {
    char *sym;
    int (*fcn)();
} ENTRY;

/* function key table structure */
typedef struct {
    char *key;
    int scode;
} KEY;

/* keyboard macro table */
char *mkey[256*8];	/* 256 entries for each combination of
			   shift/control/alt */

/* global variables */
int cecho;		/* echo commands from the script */
int mecho;		/* echo characters to/from the modem */
char *cbuf;		/* capture buffer */

/* external variables */
extern int crc;		/* TRUE for crc, FALSE for checksum */
extern int big;		/* TRUE for ymodem, FALSE for xmodem */
extern int half;	/* TRUE for half duplex, FALSE for full duplex */
extern int esc;		/* terminal mode escape character */
extern FILE *cfp;	/* capture file pointer */

/* external routines */
extern long filesize();
extern char *getenv();
extern char *envstr();
extern char *malloc();
extern FILE *fopen();
extern char *gets();

/* function key table */
KEY keys[] = {
{	"F1",		0x3B	},
{	"F2",		0x3C	},
{	"F3",		0x3D	},
{	"F4",		0x3E	},
{	"F5",		0x3F	},
{	"F6",		0x40	},
{	"F7",		0x41	},
{	"F8",		0x42	},
{	"F9",		0x43	},
{	"F10",		0x44	},
{	"CLR-HOME",	0x47	},
{	"UP",		0x48	},
{	"LEFT",		0x4B	},
{	"RIGHT",	0x4D	},
{	"DOWN",		0x50	},
{	"INSERT",	0x52	},
{	"UNDO",		0x61	},
{	"HELP",		0x62	},
{	NULL,		0	}
};

/* forward declarations for the command functions */
int CMverify(),CMnoverify(),CMwatch(),CMnowatch(),CMhalfduplex(),CMfullduplex();
int CMbaudrate(),CMtype(),CMexpect(),CMsend(),CMreceive(),CMxmodem(),CMymodem();
int CMchecksum(),CMcrc(),CMescape(),CMcall(),CMdtr(),CMnodtr(),CMmkey();
int CMcapture(),CMclose(),CMasend(),CMsystem(),CMhelp(),CMabort(),CMexit();
int CMstatus(),CMprompt(),CMset(),CMdelay();

/* command table */
ENTRY commands[] = {
{	"VERIFY",	CMverify	},
{	"NOVERIFY",	CMnoverify	},
{	"WATCH",	CMwatch		},
{	"NOWATCH",	CMnowatch	},
{	"HALF-DUPLEX",	CMhalfduplex	},
{	"FULL-DUPLEX",	CMfullduplex	},
{	"BAUD-RATE",	CMbaudrate	},
{	"PROMPT",	CMprompt	},
{	"TYPE",		CMtype		},
{	"EXPECT",	CMexpect	},
{	"SEND",		CMsend		},
{	"RECEIVE",	CMreceive	},
{	"XMODEM",	CMxmodem	},
{	"YMODEM",	CMymodem	},
{	"CHECKSUM",	CMchecksum	},
{	"CRC",		CMcrc		},
{	"ESCAPE",	CMescape	},
{	"CALL",		CMcall		},
{	"CAPTURE",	CMcapture	},
{	"CLOSE",	CMclose		},
{	"ASEND",	CMasend		},
{	"SYSTEM",	CMsystem	},
{	"ABORT",	CMabort		},
{	"EXIT",		CMexit		},
{	"DTR",		CMdtr		},
{	"NODTR",	CMnodtr		},
{	"DELAY",	CMdelay		},
{	"MKEY",		CMmkey		},
{	"SET",		CMset		},
{	"STATUS",	CMstatus	},
{	"HELP",		CMhelp		},
{	"?",		CMhelp		},
{	NULL,		NULL		}
};

/* forward declarations */
char *strsave();
char *next();

/* te_init - initialize the terminal emulator */
te_init()
{
    int i;

    /* setup the environment variables */
    if (getenv("PATH")   == NULL) insenv("PATH",DEFPATH);
    if (getenv("SUFF")   == NULL) insenv("SUFF",DEFSUFF);
    if (getenv("BMODEM") == NULL) insenv("BMODEM",DEFBMODEM);

    /* enable XON/XOFF */
    Rsconf(-1,1,-1,-1,-1,-1);

    /* clear the keyboard macro table */
    for (i = 0; i < 256*8; ++i)
	mkey[i] = NULL;

    /* install some default macros */
    mkeydef(0x3B,"half-duplex");				/* F1 */
    mkeydef(0x3C,"full-duplex");				/* F2 */
    mkeydef(0x3D,"prompt \"^M^JSend file -- \" send");		/* F3 */
    mkeydef(0x3E,"prompt \"^M^JReceive file -- \" receive");	/* F4 */
    mkeydef(0x3F,"prompt \"^M^JASCII send -- \" asend");	/* F5 */
    mkeydef(0x40,"prompt \"^M^JCapture file -- \" capture");	/* F6 */
    mkeydef(0x41,"close");					/* F7 */
    mkeydef(0x42,"prompt \"^M^J\" system");			/* F8 */
    mkeydef(0x44,"exit");					/* F10 */
    mkeydef(0x62,"prompt \"^M^J\" help");			/* help key */

    /* no capture buffer allocated */
    cbuf = NULL;
}

/* script - process a script file */
int script(sfile)
  char *sfile;
{
    char line[LSIZE],path[LSIZE],*sptr,*pp;
    FILE *fp;

    /* check each element of the path */
    for (fp = NULL, pp = getenv("BMODEM"); pp = next(pp,path); ) {

	/* insert the path separator */
	if (path[0] != '\0' && path[strlen(path)-1] != '\\')
	    strcat(path,"\\");

	/* combine the path with the command name */
	strcat(path,sfile);

	/* try opening the script file */
	if (fp = fopen(path,"r"))
	    break;
    }

    /* make sure we found the command file */
    if (fp == NULL)
	return (FALSE);

    /* initialize */
    cecho = mecho = FALSE;

    /* read and process each line from the file */
    while (sptr = fgets(line,LSIZE,fp)) {

	/* echo the command line if verify is on */
	if (cecho)
	    fputs(line,stdout);

	/* process the commands */
	if (!docommands(&sptr)) {
	    fclose(fp);
	    return (TRUE);
	}
    }

    /* close the file */
    fclose(fp);

    /* return successfully */
    return (TRUE);
}

/* expect - expect a string from the modem */
int expect(str)
  char *str;
{
    char noise[NSIZE],*np,*es,*p,*q;
    int len,ch,n;

    /* clear out the line noise buffer */
    for (np = noise, n = NSIZE; --n >= 0; )
	*np++ = '\0';

    /* initialize the string pointers */
    len = strlen(str);
    es = &str[len-1];
    np = noise;

    /* look for the string */
    for (;;) {

	/* get the next character */
	if ((ch = mdgetc(mecho)) < 0)
	    return (FALSE);

	/* strip the high bit */
	ch &= 0x7F;

	/* look for the string */
	*np = ch;
	for (p = es, q = np, n = len; --n >= 0; --p, --q) {
	    if (*p != *q)
		break;
	    if (q <= &noise[0])
		q = &noise[NSIZE];
	}

	/* exit if we found it */
	if (n < 0) break;

	/* increment the noise pointer (wrap around) */
	if (++np == &noise[NSIZE])
	    np = noise;
    }

    /* return successfully */
    return (TRUE);
}

/* mdgetc - get the next character from the modem allowing console I/O */
int mdgetc(echo)
  int echo;
{
    int scode,ch;
    long val;
    char *p;

    /* wait for a modem character */
    for (;;) {

	/* handle console input */
	if (Bconstat(2) < 0) {
	    val = Bconin(2);		/* get the character and scan code */
	    scode = (val>>16) & 0x0FF;	/* extract the scan code */
	    scode |= (val>>17) & 0x700;	/* get the modifier key status */
	    if (val & 0x01000000L)	/* combine left and right shift keys */
		scode |= 0x100;
	    ch = val & 0xFF;		/* extract the character code */
	    if (scode == esc) {		/* check for the escape character */
		if (!cmdmode())
		    return (EOF);
	    }
	    else if (p = mkey[scode])	/* check for a macro key */
		docommands(&p);
	    else {			/* handle normal characters */
		if (echo && half) {
		    Bconout(2,ch);
		    if (cfp)
			putc(ch,cfp);
		}
		Bconout(1,ch);
	    }
 	}

	/* handle modem input */
	if (Bconstat(1) < 0) {
	    ch = Bconin(1) & 0x7F;
	    if (echo) {
		Bconout(2,ch);
		if (cfp)
		    putc(ch,cfp);
	    }
	    return (ch);
	}
    }
}

/* cmdmode - handle command mode */
int cmdmode()
{
    char line[200],*sptr;

    /* process commands */
    putchar('\n');
    for (;;) {

	/* Print the command line prompt */
	printf("CMD> "); fflush(stdout);

	/* get the command line checking for EOF */
	if ((sptr = gets(line)) == NULL)
	    break;

	/* check for an empty line */
	if (!scan(&sptr,NULL))
	    break;

	/* parse the commands */
	if (!docommands(&sptr))
	    return (TRUE);
    }
    return (TRUE);
}

/* docommands - handle commands */
int docommands(sptr)
  char **sptr;
{
    char cmd[LSIZE];
    int (*fcn)();

    /* check for a system command */
    if (**sptr == '!') {
	++(*sptr);
	CMsystem(sptr);
	return (TRUE);
    }

    /* get each command and parse it */
    while (scan(sptr,cmd)) {

	/* check for a comment */
	if (cmd[0] == ';')
	    return (TRUE);

	/* lookup and execute the command */
	switch (lookup(commands,cmd,&fcn)) {
	case LS_FOUND:
	    switch ((*fcn)(sptr)) {
	    case CS_OK:
		break;
	    case CS_FLUSH:
		return (TRUE);
	    case CS_ABORT:
		return (FALSE);
	    }
	    break;
	case LS_UNKNOWN:
	    error("unknown command: %s",cmd);
	    break;
	case LS_AMBIGUOUS:
	    error("ambiguous command: %s",cmd);
	    printf("  choose one of:");
	    showmatches(commands,cmd);
	    printf("\n");
	    break;
	}
    }

    /* return successfully */
    return (TRUE);
}

int CMverify()
{
    cecho = TRUE;
    return (CS_OK);
}

int CMnoverify()
{
    cecho = FALSE;
    return (CS_OK);
}

int CMwatch()
{
    mecho = TRUE;
    return (CS_OK);
}

int CMnowatch()
{
    mecho = FALSE;
    return (CS_OK);
}

int CMhalfduplex()
{
    half = TRUE;
    return (CS_OK);
}

int CMfullduplex()
{
    half = FALSE;
    return (CS_OK);
}

int CMbaudrate(sptr)
  char **sptr;
{
    char rate[LSIZE];
    if (!getarg(sptr,"Rate? ",rate))
	return (CS_FLUSH);
    md_init(atoi(rate));
    return (CS_OK);
}

int CMprompt(sptr)
  char **sptr;
{
    char str[LSIZE],*p;
    int ch;
    if (!getarg(sptr,"String? ",str))
	return (CS_FLUSH);
    for (p = str; (ch = *p++) != '\0'; )
	Bconout(2,ch);
    return (CS_OK);
}

int CMtype(sptr)
  char **sptr;
{
    char str[LSIZE],*p;
    int ch;
    if (!getarg(sptr,"String? ",str))
	return (CS_FLUSH);
    for (p = str; (ch = *p++) != '\0'; ) {
	if (half && mecho)
	    Bconout(2,ch);
	Bconout(1,ch);
    }
    return (CS_OK);
}

int CMexpect(sptr)
  char **sptr;
{
    char str[LSIZE];
    if (!getarg(sptr,"String? ",str))
	return (CS_FLUSH);
    expect(str);
    return (CS_OK);
}

int CMsend(sptr)
  char **sptr;
{
    char file[LSIZE];
    if (!getarg(sptr,"Filename? ",file))
	return (CS_FLUSH);
    putfile(file);
    return (CS_OK);
}

int CMreceive(sptr)
  char **sptr;
{
    char file[LSIZE];
    if (!getarg(sptr,"Filename? ",file))
	return (CS_FLUSH);
    getfile(file);
    return (CS_OK);
}

int CMxmodem()
{
    big = FALSE;
    return (CS_OK);
}

int CMymodem()
{
    big = TRUE;
    return (CS_OK);
}

int CMchecksum()
{
    crc = FALSE;
    return (CS_OK);
}

int CMcrc()
{
    crc = TRUE;
    return (CS_OK);
}

int CMescape(sptr)
  char **sptr;
{
    char key[LSIZE];
    int scode;
    if (!getarg(sptr,"Key code? ",key))
	return (CS_FLUSH);
    if ((scode = decode_key(key)) < 0) {
	error("unknown key name: %s",key);
	return (CS_FLUSH);
    }    
    esc = scode;
    return (CS_OK);
}

int CMcall(sptr)
  char **sptr;
{
    char file[LSIZE];
    if (!getarg(sptr,"Filename? ",file))
	return (CS_FLUSH);
    if (!script(file))
	error("can't open script file: %s",file);
    return (CS_OK);
}

int CMcapture(sptr)
  char **sptr;
{
    char file[LSIZE];
    if (!getarg(sptr,"Filename? ",file))
	return (CS_FLUSH);
    if (cfp) fclose(cfp);
    if ((cfp = fopen(file,"wb")) == NULL)
	error("can't open capture file: %s",file);
    if (cbuf == NULL)
	cbuf = malloc(CSIZE);
    if (cbuf)
	setbuf(cfp,cbuf);
    return (CS_OK);
}

int CMclose()
{
    if (cfp) {
	fclose(cfp);
	cfp = NULL;
    }
    return (CS_OK);
}

int CMasend(sptr)
  char **sptr;
{
    char file[LSIZE];
    int lineno,ch;
    long fsize;
    FILE *fp;
    if (!getarg(sptr,"Filename? ",file))
	return (CS_FLUSH);
    if ((fsize = filesize(file)) < 0L) {
	error("can't access file: %s",file);
	return (CS_FLUSH);
    }
    if ((fp = fopen(file,"rb")) == NULL) {
	error("can't open ASCII upload file: %s",file);
	return (CS_FLUSH);
    }
    printf("[ %s: %ld bytes ]\n",file,fsize);
    for (lineno = 0, fsize = 0L; (ch = getc(fp)) >= 0; ++fsize)
	if (ch != '\0' && ch != '\n') {
	    Bconout(1,ch);
	    if (ch == '\r') {
		printf("\rSending line %d, %ld bytes",++lineno,fsize);
		fflush(stdout);
		for (;;) {
		    if (Bconstat(2) < 0) {	/* handle console input */
			ch = Bconin(2) & 0x7F;
			if (ch == '\n')
			   break;
			else if (ch == '\033')
			   goto done;
	 	    }
		    if (Bconstat(1) < 0) {	/* handle modem input */
			ch = Bconin(1) & 0x7F;
			if (ch == '\n')
			    break;
		    }
		}
		while (Bconstat(1) < 0)		/* flush the modem buffer */
		    Bconin(1);
	    }
	}
done:
    fclose(fp);
    printf("\007\n[ ASCII send complete ]\n");
    return (CS_OK);
}

int CMdtr()
{
    Offgibit(0xFFEF);
    return (CS_OK);
}

int CMnodtr()
{
    Ongibit(0x10);
    return (CS_OK);
}

int CMmkey(sptr)
  char **sptr;
{
    char key[LSIZE],str[LSIZE];
    int scode;
    if (!getarg(sptr,"Key code? ",key) || !getarg(sptr,"String? ",str))
	return (CS_FLUSH);
    if ((scode = decode_key(key)) < 0) {
	error("unknown key name: %s",key);
	return (CS_FLUSH);
    }    
    mkeydef(scode,str);
    return (CS_OK);
}

int CMsystem(sptr)
  char **sptr;
{
    char line[LSIZE],cmd[LSIZE],path[LSIZE],ext[LSIZE],args[LSIZE];
    char *pp,*ep,*p;
    short int sts;
    int pathlen;

    /* skip leading spaces */
    while (**sptr && isspace(**sptr))
	++(*sptr);

    /* if the line is empty, prompt for a command */
    if (**sptr == '\0') {
	printf("Command? ");
	if (gets(line) == NULL || line[0] == '\0')
	    return (CS_FLUSH);
	*sptr = line;
    }

    /* get the command name */
    for (p = cmd; **sptr && !isspace(**sptr); )
	*p++ = *(*sptr)++;
    *p = '\0';

    /* skip spaces between the command name and the arguments */
    while (**sptr && isspace(**sptr))
	++(*sptr);

    /* make a counted ascii argument list */
    for (p = &args[1], args[0] = '\0'; **sptr; ++args[0])
	*p++ = *(*sptr)++;
    *p = '\0';

    /* check each element of the path */
    for (pp = getenv("PATH"); pp = next(pp,path); ) {

	/* skip over the built-in commands */
	if (strcmp(path,".bin") == 0)
	    continue;

	/* insert the path separator */
	if (path[0] != '\0')
	    strcat(path,"\\");

	/* combine the path with the command name */
	strcat(path,cmd);

	/* determine the length of the path so far */
	pathlen = strlen(path);

	/* check each extension */
	for (ep = getenv("SUFF"); ep = next(ep,ext); ) {

	    /* add the extension to the path */
	    strcpy(&path[pathlen],ext);

	    /* try to execute the command */
	    if ((sts = Pexec(0,path,args,envstr())) != -33)
		return (CS_OK);
	}
    }

    /* can't find the program */
    error("can't execute: %s",cmd);
    return (CS_FLUSH);
}

int CMabort()
{
    return (CS_ABORT);
}

int CMexit()
{
    cleanup();
}

CMset(sptr)
  char **sptr;
{
    char name[LSIZE],value[LSIZE];
    if (!getarg(sptr,"Variable? ",name))
	return (CS_FLUSH);
    if (!getarg(sptr,"Value? ",value))
	return (CS_FLUSH);
    insenv(name,value);
    return (CS_OK);
}

int CMstatus()
{
    printf("%s duplex\n",(half ? "Half" : "Full"));
    printf("VERIFY is %s\n",(cecho ? "on" : "off"));
    printf("WATCH is %s\n",(mecho ? "on" : "off"));
    printf("PATH is %s\n",getenv("PATH"));
    printf("SUFF is %s\n",getenv("SUFF"));
    printf("BMODEM is %s\n",getenv("BMODEM"));
    printf("Capture file is %s\n",(cfp ? "open" : "closed"));
    printf("%sMODEM",(big ? "Y" : "X"));
    printf("/%s transfer mode\n",(crc ? "CRC" : "CHECKSUM"));
}

int CMdelay(sptr)
  char **sptr;
{
    char amount[LSIZE];
    int n;
    if (!getarg(sptr,"Amount (tenths of seconds)? ",amount))
	return (CS_FLUSH);
    n = atoi(amount) * HZ / 10;
    while (--n >= 0)
	Vsync();
    return (CS_OK);
}

int CMhelp(sptr)
  char **sptr;
{
    char cmd[LSIZE];
    if (scan(sptr,cmd) && strcmp(cmd,"MORE") == 0)
	CMmorehelp();
    else
	printf("\
TYPE str	-- send a string to the modem\n\
EXPECT str	-- expect a string from the modem\n\
PROMPT str      -- send a string to the console\n\
SEND file	-- send a file\n\
RECEIVE file	-- receive a file\n\
XMODEM/YMODEM	-- use XMODEM/YMODEM protocol for transfers\n\
CHECKSUM/CRC	-- use checksum/crc error checking for transfers\n\
VERIFY/NOVERIFY -- turn on/off echoing of commands from a script\n\
WATCH/NOWATCH	-- turn on/off watching of modem data during a script\n\
DTR/NODTR	-- turn on/off Data Terminal Ready\n\
FULL-DUPLEX	-- use full duplex (no local echo)\n\
HALF-DUPLEX	-- use half duplex (local echo)\n\
BAUD-RATE rate	-- set the baud rate\n\
ESCAPE char	-- set the terminal mode escape character\n\
CALL script     -- invoke a script file\n\
CAPTURE file    -- open a file to capture the terminal session\n\
CLOSE           -- close the current capture file\n\
ASEND file      -- send an ASCII file with no protocol\n\
MKEY key str    -- define a macro key (str is a command string)\n\
SYSTEM cmd      -- invoke an external program (short form is !)\n\
SET var value   -- set the value of an environment variable\n\
STATUS          -- show program status\n\
HELP [more]     -- print help text (or more help text)\n\
ABORT/EXIT      -- abort the current script/exit from BMODEM\n\
");
    return (CS_OK);
}

CMmorehelp()
{
    printf("\
Default key bindings:\n\
    F1   = half-duplex\n\
    F2   = full-duplex\n\
    F3   = prompt \"^M^JSend file -- \" send\n\
    F4   = prompt \"^M^JReceive file -- \" receive\n\
    F5   = prompt \"^M^JASCII send -- \" asend\n\
    F6   = prompt \"^M^JCapture file -- \" capture\n\
    F7   = close\n\
    F8   = prompt \"^M^J\" system\n\
    F10  = exit\n\
    HELP = prompt \"^M^J\" help\n\
\n\
Acceptable key names are:\n\
    F1 F2 F3 F4 F5 F6 F7 F8 F9 F10 HELP UNDO INSERT CLR-HOME\n\
    UP DOWN LEFT RIGHT (any hex key code preceeded by '$')\n\
\n\
Note:  Parameters can be quoted strings.  Use double quotes and escape any\n\
embedded double quotes with backslashes.  Control characters are represented\n\
using the caret character (for example, ^M is CTRL-M).  If you need to\n\
include a caret in a string, escape it with a backslash also.  Here is a\n\
complete example:  TYPE \"^Hbix^M\".\n\
");
    return (CS_OK);
}

/* mkeydef - define a macro key */
mkeydef(scode,string)
  int scode; char *string;
{
    scode &= 0x7FF;
    if (mkey[scode])
	free(mkey[scode]);
    mkey[scode] = strsave(string);
}

/* getarg - get an argument (prompt if necessary) */
int getarg(sptr,prompt,arg)
  char **sptr,*prompt,*arg;
{
    if (scan(sptr,arg))
	return (TRUE);
    printf(prompt);
    return (gets(arg) == NULL || arg[0] == '\0' ? FALSE : TRUE);
}

/* scan - command line parsing routine */
int scan(sptr,arg)
  char **sptr,*arg;
{
    int ch;

    /* skip leading spaces */
    while (**sptr != '\0' && isspace(**sptr))
	++(*sptr);

    /* make sure there is an argument present */
    if (**sptr == '\0')
	return (FALSE);

    /* return immediately if there is no argument to parse */
    if (arg == NULL)
	return (TRUE);

    /* handle quoted strings */
    if (**sptr == '"') {
	for (++(*sptr); **sptr != '\0' && (ch = *(*sptr)++) != '"'; ) {
	    if (ch == '\\' && **sptr != '\0')
		ch = *(*sptr)++;
	    else if (ch == '^' && **sptr != '\0') {
		ch = (islower(**sptr) ? toupper(**sptr) : **sptr) - 0x40;
		++(*sptr);
	    }
	    *arg++ = ch;
	}
    }

    /* handle normal tokens */
    else {
	for (; **sptr != '\0' && !isspace(**sptr); ++(*sptr))
	    *arg++ = (islower(**sptr) ? toupper(**sptr) : **sptr);
    }

    /* terminate the string */
    *arg = '\0';

    /* return successfully */
    return (TRUE);
}

/* lookup - lookup a token in a symbol table */
int lookup(table,token,pfcn)
  ENTRY *table; char *token; int (**pfcn)();
{
    char *s,*t;
    ENTRY *e;
    int val;

    /* initialize */
    val = LS_UNKNOWN;

    /* search each table entry */
    for (e = table; (s = e->sym) != NULL; ++e) {
	for (t = token; *s != '\0' && *t != '\0'; ++s, ++t)
	    if (*s != *t)
		break;
	if (*t == '\0') {
	    if (*s == '\0') {
		*pfcn = e->fcn;
		return (LS_FOUND);
	    }
	    else if (val == LS_UNKNOWN) {
		*pfcn = e->fcn;
		val = LS_FOUND;
	    }
	    else
		return (LS_AMBIGUOUS);
	}
    }

    /* return the result */
    return (val);
}

/* showmatches - show partial matches to a token */
showmatches(table,token)
  ENTRY *table; char *token;
{
    char *s,*t;
    ENTRY *e;

    /* search each table entry */
    for (e = table; (s = e->sym) != NULL; ++e) {
	for (t = token; *s != '\0' && *t != '\0'; ++s, ++t)
	    if (*s != *t)
		break;
	if (*t == '\0')
	    printf(" %s",e->sym);
    }
}

/* next - get the next element of a comma delimited list */
char *next(list,elem)
  char *list,*elem;
{
    /* check for the end of the list */
    if (*list == '\0')
	return (NULL);

    /* get the next element from the list */
    while (*list != '\0' && *list != ',')
	*elem++ = *list++;
    *elem = '\0';

    /* skip past the comma */
    if (*list == ',')
	++list;

    /* return the tail of the list */
    return (list);
}

/* strsave - create a copy of a string */
char *strsave(str)
  char *str;
{
    char *cpy;
    if (cpy = malloc(strlen(str)+1))
	strcpy(cpy,str);
    return (cpy);
}

/* decode_key - decode a key name */
int decode_key(key)
  char *key;
{
    int scode,i;
    char *pkey;

    /* initialize */
    pkey = key;
    scode = 0;

    /* handle prefixes */
    for (;;) {
	if (strncmp(pkey,"S-",2) == 0)
	    { scode |= 0x100; pkey += 2; }
	else if (strncmp(pkey,"C-",2) == 0)
	    { scode |= 0x200; pkey += 2; }
	else if (strncmp(pkey,"A-",2) == 0)
	    { scode |= 0x400; pkey += 2; }
	else
	    break;
    }

    /* check for a hex keycode */
    if (*pkey == '$')
	return (scode | xtoi(++pkey));

    /* lookup the key name */
    for (i = 0; keys[i].key; ++i)
	if (strcmp(pkey,keys[i].key) == 0) {
	    scode |= keys[i].scode;
	    return (scode);
	}

    /* unknown key name */
    return (-1);
}

/* xtoi - convert a hex number to an integer */
int xtoi(str)
  char *str;
{
    int num,ch;
    for (num = 0; (ch = *str); ++str)
	if (ch >= '0' && ch <= '9')
	    num = (num << 4) + ch - '0';
	else if (ch >= 'A' && ch <= 'F')
	    num = (num << 4) + ch - ('A' - 10);
	else if (ch >= 'a' && ch <= 'f')
	    num = (num << 4) + ch - ('a' - 10);
	else
	    break;
    return (num);
}

/* error - print an error message and return FALSE */
int error(msg,arg)
  char *msg,*arg;
{
    printf("Error: ");
    printf(msg,arg);
    printf("\n");
    return (FALSE);
}
